@@ -22,16 +22,18 @@ class ComponentOutput:
   def __init__(self, app):
     self._app = app
 
-  def build_node(self, name, columns, relation, scope):
-    self._name = name
-    self._type = "output"
-    self._columns = columns
+  def build_node(self, **kwargs):
 
+    self._name = kwargs.get("name")
+    self._type = "output"
+    self._columns = kwargs.get("fields")
+    
+    relation = kwargs.get("relation")
     if relation != None:
       self._relation = relation
     else:
-      self._relation = name
-    self._scope = scope
+      self._relation = self._name
+    self._scope = kwargs.get("scope")
 
     Helper.check_sink(self._name, self._columns, self._app._nodes, "outputs")
 
