@@ -1,19 +1,20 @@
 import re
 import sys
 import json
+import inspect
 
 class Helper:
 
   ALLOWED_END_CYCLE_POLICIES= ["null_emit", "explicit"]
   ALLOWED_OUTPUT_FORMATS = ["replace", "merge"]
   ALLOWED_TYPES = ["string", "integer", "float", "double", "boolean", "array", "map"]
- 
+
   argv = sys.argv[1:]
   @classmethod
   def set_argv(self, v):
     self.argv = v
 
-  @classmethod  
+  @classmethod
   def opt_parser(self):
     options = {}
     i = 0
@@ -37,12 +38,13 @@ class Helper:
         options["port"] = self.argv[i]
       i += 1
     return options
-  
+
+
   @classmethod
   def print_error(self, msg):
     sys.stderr.write(msg+"\n")
     sys.exit(1)
- 
+
   @classmethod
   def write_hash_to_file(self, h, f):
     f.write(json.dumps(h)+"\n")
@@ -55,11 +57,11 @@ class Helper:
   @classmethod
   def write_arc_to_file(self, h, f):
     self.write_hash_to_file({"class":"arc", "info":h},f)
- 
+
   @classmethod
   def check_name(self, operation, name, names):
     ee = "Error in \"%s\" at \"name\": \n\t " % (operation)
-  
+
     if not isinstance(name, basestring) or re.match('^[\w]+$', name) == None:
       msg = ee+"\"Name\" must be a non-empty STRING with only alphanumeric and underscore characters at %s. Apps and components must have names. The following methods must have names as well: \"sink\", \"inputs\" and \"outputs\"."%(str(name))
       self.print_error(msg)
@@ -131,14 +133,14 @@ class Helper:
     if((not rm) and (not mm) and (not node._next_tuple)):
       msg = ee+"A \"source\" must contain a valid \"matches\" argument or a \"next_tuple\" argument."+pp
       self.print_error(msg)
- 
+
   @classmethod
   def check_each(self, node):
     ee = "Error in \"each\": \n\t "
     pp = self.print_check_each
 
     if (node._parallelism != None and not (type(node._parallelism) is int and node._parallelism > 0)):
-      msg = "Parallelism for \"each\" must be positive integer" 
+      msg = "Parallelism for \"each\" must be positive integer"
       self.print_error(msg)
 
     if(node._execute == None):
@@ -157,7 +159,7 @@ class Helper:
       self.print_error(msg)
 
     if (node._parallelism != None and not (type(node._parallelism) is int and node._parallelism > 0)):
-      msg = "Parallelism for \"each\" must be positive integer" 
+      msg = "Parallelism for \"each\" must be positive integer"
       self.print_error(msg)
 
     if(node._keep == None):
@@ -237,28 +239,28 @@ class Helper:
       pp = self.print_check_sink
     elif operation == "outputs":
       pp = self.print_check_component_sink
-  
+
     if not name:
       msg = ee+"Relation name must be specified!"+pp
       self.print_error(msg)
     self.check_name(operation, name, {})
-  
+
     if len(columns) == 0:
       msg = ee+"Must be at least one output field to relation \"%s\"."%(name)+pp
       self.print_error(msg)
     self.check_field_format(operation, pp, name, columns)
-  
+
     for node in nodes:
       if node._type != "sink":
         continue
       if node._name == name and node._columns != columns:
         msg = ee+"The relation \"%s\" has already been specified and contains a different set of fields/types."%(name)+pp
         self.print_error(msg)
-  
+
   @classmethod
   def check_field_format(self, operation, pp, name, columns):
     ee = "Error in \"%s\": \n\t "%(operation)
-  
+
     if not isinstance(columns, list):
       msg = ee+"Field names must be a LIST of DICTIONARIES in \"%s\"."%(name)+pp
       self.print_error(msg)
@@ -288,7 +290,7 @@ class Helper:
     pp = self.print_check_component_source
 
     self.check_field_format("inputs", pp, node._name, node._fields)
-  
+
   @classmethod
   def check_component_sink(self, node):
     ee = "Error in \"outputs\": \n\t "
@@ -324,29 +326,60 @@ class Helper:
   print_check_source = """\n
 "Source" Syntax:
   Sourcing from a relation:
-    app.source( matches = "SQL query" or [SXP queries] )
+    app.source("relation_name" )
 
-  Custom source:
-    app.source( name = "name", \t\t\t\t\t => optional
+  Simple Custom source:
+    def next_tuple(controller):
+        # next_tuple_function
+
+    app.source(next_tuple_function)
+
+
+  Extended Source Syntax:
+
+    class Source:
+        def next_tuple(self, controller):
+            #next_tuple_method
+        def begin_cycle(self, controller):
+            #begin_cycle_method
+
+    source = app.source(Source,
+                name = "name", \t\t\t\t\t => optional
                 emits = ["stream_1", "stream_2", ...], \t\t => optional for single output stream
                 end_cycle_policy = "null_emit" OR "explicit", \t => default "null_emit"
-                begin_cycle = begin_cycle_function, \t\t => optional if no initialization needed
-                next_tuple = next_tuple_function )
 
   - The "end_cycle_policy" is used to specify when a cycle should end. Two options are available:
       * :null_emit - end the cycle when a field contains "nil" or when nothing is emitted from the "next_tuple" block.
       * :explicit - the end of a cycle is explicitly declared in the "next_tuple" block. This is done by including the "end_cycle" keyword in the "next_tuple" function, e.g. end_cycle() if queue.empty.
-  - The "begin_cycle_function" and "next_tuple_function" can be full functions or lambda functions. 
+  - The "begin_cycle_function" and "next_tuple_function" can be full functions or lambda functions.
       * "begin_cycle_function" must take in a single argument (the "controller") and should return nothing. This is where any setup is done to initialize the content and quantity of tuples emitted by "next_tuple_function".
       * "next_tuple_function" must take in 2 arguments (the "controller" and the "tuple"), and should return nothing. This is where the tuples are actually emitted."""
 
   print_check_each = """\n
 "Each" Syntax:
-  stream.each( name = "name", \t\t\t\t => optional
+
+  Simple Syntax:
+
+    def execute(controller, tuple):
+        # execute_function
+
+    stream.each(execute)
+
+  Extended Syntax:
+
+    class EachClass:
+        def prepare(self, controller):
+          # prepare_method
+        def execute(self, controller, tup):
+          # execcute_method
+
+  stream.each( EachClass,
+               name = "name", \t\t\t\t => optional
                emits = ["stream_1", "stream_2", ...], \t => optional for single output stream
                output_format = "replace" OR "merge", \t => optional, defaults to "replace"
                prepare = prepare_function, \t\t => optional if no initialization needed
                execute = execute_function )
+
   - The allowed output formats are "replace" and "merge".
       * "replace" - discards the input tuple values and only emits the specified values. This is the default.
       * "merge" - re-emits the input tuple values along with the specified values.
@@ -355,7 +388,25 @@ class Helper:
 
   print_check_filter = """\n
 "Filter" Syntax:
-  stream.filter( name = "name", \t\t\t\t => optional
+
+  Simple Syntax:
+
+    def keep(tup):
+      # keep_function
+
+    stream.keep(keep)
+
+  Extended Syntax:
+    class FilterClass:
+      def prepare(self, controller):
+        # prepare_method
+
+      def keep(self, tup):
+        # keep_method
+
+
+  stream.filter( FilterClass,
+                 name = "name", \t\t\t\t => optional
                  emits = "stream", \t\t\t\t => optional
                  prepare = prepare_function, \t => optional if no initialization needed
                  keep = keep_function )
@@ -365,15 +416,23 @@ class Helper:
 
   print_check_group_by = """\n
 "Group By" Syntax:
-  stream.group_by( name = "name", \t\t\t\t => optional
+
+  class GroupByClass:
+    def begin_group(self, controller, group_tuple):
+      # begin_group_method
+    def aggregate(self, controller, tup):
+      # aggregate_method
+    def end_group(self, controller):
+      # end_group_method
+
+
+  stream.group_by( GroupByClass,
+                   name = "name", \t\t\t\t => optional
                    fields = ["field_1", "field_2", ...],
-                   emits = ["stream_1", "stream_2", ...], \t => optional for single output stream
-                   begin_group = begin_group_function
-                   aggregate = aggregate_function
-                   end_group = end_group_function )
-  - The "begin_group_function" must take in 2 arguments (the "controller" and the "grouping tuple", which is emitted at the beginning of each group and contains the values of the fields specified in "group_by"), and should return nothing. This is where initial values for the aggregation are set.
-  - The "aggregate_function" must take in 2 arguments (the "controller" and the "tuple"), and should return nothing. This is where the aggregation is performed.
-  - The "end_group_function" must take in a single argument (the "controller"), and should return nothing. This is where the final aggregated value is emitted."""
+                   emits = ["stream_1", "stream_2", ...], \t => optional for single output stream )
+  - The "begin_group_method" must take in 3 arguments (the "self", the "controller", and the "grouping tuple", which is emitted at the beginning of each group and contains the values of the fields specified in "group_by"), and should return nothing. This is where initial values for the aggregation are set.
+  - The "aggregate_method" must take in 3 arguments (the "self", the "controller", and the "tuple"), and should return nothing. This is where the aggregation is performed.
+  - The "end_group_method" must take in a 2 arguments (the "self" and the "controller"), and should return nothing. This is where the final aggregated value is emitted."""
 
   print_check_join = """\n
 "Join" Syntax:
@@ -396,7 +455,7 @@ class Helper:
   - Field names must be non-empty STRINGS with only alphanumeric or underscore characters.
   - Field names cannot be "v[number]", "id", "confidence", "since" or "source" which are reserved Zillabyte names.
   - Field types may be "string", "integer", "float", "double", "boolean", "array" or "map"."""
-  
+
   print_check_component_sink = """\n
 "Outputs" Syntax:
   stream.outputs( name = "output_stream_name",
@@ -416,7 +475,7 @@ class Helper:
   - Field names must be non-empty STRINGS with only alphanumeric or underscore characters.
   - Field names cannot be "v[number]", "id", "confidence", "since" or "source" which are reserved Zillabyte names.
   - Field types may be "string", "integer", "float", "double", "boolean", "array" or "map"."""
-  
+
   print_check_component_sink = """\n
 "Outputs" Syntax:
   component_stream.outputs( name = "output_stream_name",
