@@ -22,16 +22,12 @@ class Sink:
   def __init__(self, app):
     self._app = app
 
-  def build_node(self, name, columns, relation, scope):
-    self._name = name
+  def build_node(self, **kwargs):
+    self._name = kwargs.get("name", None)
     self._type = "sink"
-    self._columns = columns
-
-    if relation != None:
-      self._relation = relation
-    else:
-      self._relation = name
-    self._scope = scope
+    self._columns =  kwargs.get("columns", None)
+    self._relation = kwargs.get("relation", self._name)
+    self._scope = kwargs.get("scope", None)
 
     Helper.check_sink(self._name, self._columns, self._app._nodes, "sink")
 
